#!/usr/bin/ruby1.8
#******************************************************************************
# Module:
#	ActiveObject
#
# Description:
#	Implements CRegion style serialized event type message processing
#	through network connections.
#
#	No messages should be sent directly to this object or sub-classes
#	of it.  Instead, messages should be marshaled and enqueued for
#	execution.
#
# Author:
#	John W. M. Stevens
#******************************************************************************

class ActiveObject

	#--------------------------------------------------------------------------
	# Method:
	#	initialize
	#
	# Parameters:
	#	None.
	#
	# Description:
	#	Initialize ActiveObject, saving the read channel object from which
	#	messages will be read, and turning it on.
	#
	# Returns:
	#	Initialized object.
	#--------------------------------------------------------------------------

	def initialize(
		readChannel )

		#	Set go flag to true.
		@channel = readChannel
		@goNow   = true
	end

	#--------------------------------------------------------------------------
	# Method:
	#	stopMeBeforeIGoAroundAgain
	#
	# Parameters:
	#	None.
	#
	# Description:
	#	Set stop processing flag.
	#
	# Returns:
	#	None.
	#--------------------------------------------------------------------------

	def stopMeBeforeIGoAroundAgain
		@goNow = false
	end

	#--------------------------------------------------------------------------
	# Method:
	#	processMessagesUntilStopped
	#
	# Parameters:
	#	None.
	#
	# Description:
	#	Execute messages in queue in first in, first out order.  All
	#	messages must be event type messages.  To enforce this, all
	#	return values and side effects will be disposed of.
	#
	#	If the message queue is empty, wait for a message to be
	#	queued.
	#
	# Returns:
	#	None.
	#--------------------------------------------------------------------------

	def processMessagesUntilStopped
		#	Process messages until stopped.
		while @goNow
			begin
				#	Get message payload from socket.
				data          = @channel.read( 4 )
				messageLength = data.unpack('l').shift
				payload       = @channel.read( messageLength )

				#	Unpack message.
				message, arguments =
					Marshal.load(
						payload )

				#	Send message to self.
				self.__send__(
					message,
					*arguments )
			rescue
				#	Remove proxy from receptionist.
				$stderr.print "Uncaught Exception: #{$!}\n"
				exit 1
			end
		end
	end
end
